const { prepare, request, getResult, getCardList } = KlipGlobal

// HTML ELEMENTS
const textArea = document.querySelector('.text-area')
const prepareAuthBtn = document.querySelector('.prepare-auth-btn')
const prepareSendKLAYBtn = document.querySelector('.prepare-send-klay-btn')
const prepareSendTokenBtn = document.querySelector('.prepare-send-token-btn')
const prepareSendCardBtn = document.querySelector('.prepare-send-card-btn')
const prepareExecuteContractBtn = document.querySelector('.prepare-execute-contract-btn')
const requestBtn = document.querySelector('.request-btn')
const getCardListBtn = document.querySelector('.get-cards-btn')
const resultBtn = document.querySelector('.result-btn')

const authModal = document.querySelector('.auth')
const sendKLAYModal = document.querySelector('.send-klay')
const sendTokenModal = document.querySelector('.send-token')
const sendCardModal = document.querySelector('.send-card')
const executeContractModal = document.querySelector('.execute-contract')
const getCardModal = document.querySelector('.get-cards')

let requestKey

// HELPER FUNCTIONS
const setRequestKey = (newRequestKey) => {
  requestKey = newRequestKey
  textArea.textContent = `Request Key: ${newRequestKey}`
}

const setErrorMsg = (errorMsg) => {
  textArea.textContent = `Error: ${errorMsg}`
}

const configureModal = (modal, prepareFunction) => {
  const backdrop = modal.querySelector('.backdrop')
  const cancelBtn = modal.querySelector('.cancel-btn')
  const confirmBtn = modal.querySelector('.confirm-btn')
  const closeModal = () => {
    modal.classList.add('hidden')
  }
  backdrop.addEventListener('click', closeModal)
  cancelBtn.addEventListener('click', closeModal)
  confirmBtn.addEventListener('click', () => {
    prepareFunction()
    closeModal()
  })
}

// SET AUTH REQUEST
prepareAuthBtn.addEventListener('click', () => {
  authModal.classList.remove('hidden')
})

const authPrepare = async () => {
  const bappName = authModal.querySelector('input[name="name"]').value
  const successLink = authModal.querySelector('input[name="success-link"]').value
  const failLink = authModal.querySelector('input[name="fail-link"]').value

  const res = await prepare.auth({ bappName, successLink, failLink })
  if (res.err) {
    setErrorMsg(res.err)
  } else {
    setRequestKey(res.request_key)
  }
}

configureModal(authModal, authPrepare)

// SET SEND KLAY REQUEST
prepareSendKLAYBtn.addEventListener('click', () => {
  sendKLAYModal.classList.remove('hidden')
})

const sendKLAYPrepare = async () => {
  const bappName = sendKLAYModal.querySelector('input[name="name"]').value
  const from = sendKLAYModal.querySelector('input[name="from"]').value
  const to = sendKLAYModal.querySelector('input[name="to"]').value
  const amount = sendKLAYModal.querySelector('input[name="amount"]').value
  const successLink = sendKLAYModal.querySelector('input[name="success-link"]').value
  const failLink = sendKLAYModal.querySelector('input[name="fail-link"]').value

  const res = await prepare.sendKLAY({
    bappName,
    from,
    to,
    amount,
    successLink,
    failLink,
  })
  if (res.err) {
    setErrorMsg(res.err)
  } else {
    setRequestKey(res.request_key)
  }
}

configureModal(sendKLAYModal, sendKLAYPrepare)

// SET SEND TOKEN REQUEST
prepareSendTokenBtn.addEventListener('click', () => {
  sendTokenModal.classList.remove('hidden')
})

const sendTokenPrepare = async () => {
  const bappName = sendTokenModal.querySelector('input[name="name"]').value
  const from = sendKLAYModal.querySelector('input[name="from"]').value
  const to = sendTokenModal.querySelector('input[name="to"]').value
  const amount = sendTokenModal.querySelector('input[name="amount"]').value
  const contract = sendTokenModal.querySelector('input[name="contract"]').value
  const successLink = sendTokenModal.querySelector('input[name="success-link"]').value
  const failLink = sendTokenModal.querySelector('input[name="fail-link"]').value

  const res = await prepare.sendToken({
    bappName,
    from,
    to,
    amount,
    contract,
    successLink,
    failLink,
  })
  if (res.err) {
    setErrorMsg(res.err)
  } else {
    setRequestKey(res.request_key)
  }
}

configureModal(sendTokenModal, sendTokenPrepare)

// SET SEND CARD REQUEST
prepareSendCardBtn.addEventListener('click', () => {
  sendCardModal.classList.remove('hidden')
})

const sendCardPrepare = async () => {
  const bappName = sendCardModal.querySelector('input[name="name"]').value
  const from = sendKLAYModal.querySelector('input[name="from"]').value
  const to = sendCardModal.querySelector('input[name="to"]').value
  const id = sendCardModal.querySelector('input[name="id"]').value
  const contract = sendCardModal.querySelector('input[name="contract"]').value
  const successLink = sendCardModal.querySelector('input[name="success-link"]').value
  const failLink = sendCardModal.querySelector('input[name="fail-link"]').value

  const res = await prepare.sendCard({
    bappName,
    from,
    to,
    id,
    contract,
    successLink,
    failLink,
  })
  if (res.err) {
    setErrorMsg(res.err)
  } else {
    setRequestKey(res.request_key)
  }
}

configureModal(sendCardModal, sendCardPrepare)

// SET EXECUTE CONTRACT REQUEST
prepareExecuteContractBtn.addEventListener('click', () => {
  executeContractModal.classList.remove('hidden')
})

const executeContractPrepare = async () => {
  const bappName = executeContractModal.querySelector('input[name="name"]').value
  const from = sendKLAYModal.querySelector('input[name="from"]').value
  const to = executeContractModal.querySelector('input[name="to"]').value
  const value = executeContractModal.querySelector('input[name="value"]').value
  const abi = executeContractModal.querySelector('input[name="abi"]').value
  const params = executeContractModal.querySelector('input[name="params"]').value
  const successLink = executeContractModal.querySelector('input[name="success-link"]').value
  const failLink = executeContractModal.querySelector('input[name="fail-link"]').value

  const res = await prepare.executeContract({
    bappName,
    from,
    to,
    value,
    abi,
    params,
    successLink,
    failLink,
  })
  if (res.err) {
    setErrorMsg(res.err)
  } else {
    setRequestKey(res.request_key)
  }
}

configureModal(executeContractModal, executeContractPrepare)

// GET CARDS REQUEST
getCardListBtn.addEventListener('click', () => {
  getCardModal.classList.remove('hidden')
})

const getCardListRequest = async () => {
  const contract = getCardModal.querySelector('input[name="contract"]').value
  const eoa = getCardModal.querySelector('input[name="eoa"]').value

  const res = await getCardList({ contract, eoa })
  console.log({ res })
  if (res.err) {
    setErrorMsg(res.err)
  } else {
    textArea.textContent = `Result: ${JSON.stringify(res)}`
  }
}

configureModal(getCardModal, getCardListRequest)

// REQUEST BUTTON
requestBtn.addEventListener('click', (e) => {
  alert(`sending request with key: ${requestKey}`)
  request(requestKey)
})

// RESULT BUTTON
resultBtn.addEventListener('click', async (e) => {
  const res = await getResult(requestKey)
  textArea.textContent = `Result: ${JSON.stringify(res)}`
})
